@@ -88,6 +88,9 @@ class ImageElement(VideoBase):
             # Apply border and background
             img = self._apply_border_and_background_to_image(img)
             
+            # Apply blur effect
+            img = self._apply_blur_to_image(img)
+            
             # Update texture size
             self.texture_width, self.texture_height = img.size
             
@@ -207,36 +210,40 @@ class ImageElement(VideoBase):
         if self.texture_id is None or not self.texture_created:
             return
         
+        # Update animated properties first
+        self.update_animated_properties(time)
+        
         # Get animated properties
         animated_props = self.get_animated_properties(time)
-        
-        # Calculate current position and scale with animations
-        current_scale = animated_props.get('scale', 1.0)
         current_alpha = animated_props.get('alpha', 1.0)
         
-        # Calculate scaled dimensions
-        scaled_width = self.texture_width * current_scale
-        scaled_height = self.texture_height * current_scale
+        # Get actual render position using the fixed VideoBase method
+        # This already considers scale in the anchor offset calculation
+        render_x, render_y, scaled_width, scaled_height = self.get_actual_render_position()
         
-        # Get actual render position using anchor calculation
-        # Temporarily set the current size for anchor calculation
-        original_width, original_height = self.width, self.height
-        self.width, self.height = scaled_width, scaled_height
+        # Save current OpenGL matrix state
+        glPushMatrix()
         
-        # Get position with anchor offset applied
-        render_x, render_y, _, _ = self.get_actual_render_position()
+        # Apply transformations (rotation and scale) around the center
+        center_x = render_x + scaled_width / 2
+        center_y = render_y + scaled_height / 2
         
-        # Apply animation offsets to the anchor-adjusted position
-        if 'x' in animated_props:
-            render_x = animated_props['x'] + self._calculate_anchor_offset(scaled_width, scaled_height)[0]
-        if 'y' in animated_props:
-            render_y = animated_props['y'] + self._calculate_anchor_offset(scaled_width, scaled_height)[1]
-            
-        # Restore original size
-        self.width, self.height = original_width, original_height
+        # Move to center for rotation
+        glTranslatef(center_x, center_y, 0)
         
-        # Save current OpenGL state
-        glPushAttrib(GL_ALL_ATTRIB_BITS)
+        # Apply rotation if set
+        current_rotation = animated_props.get('rotation', getattr(self, 'rotation', 0.0))
+        if current_rotation != 0:
+            glRotatef(current_rotation, 0, 0, 1)
+        
+        # Apply flip transformations
+        flip_x = -1.0 if getattr(self, 'flip_horizontal', False) else 1.0
+        flip_y = -1.0 if getattr(self, 'flip_vertical', False) else 1.0
+        if flip_x != 1.0 or flip_y != 1.0:
+            glScalef(flip_x, flip_y, 1.0)
+        
+        # Move back from center
+        glTranslatef(-center_x, -center_y, 0)
         
         # Apply alpha
         if current_alpha < 1.0:
@@ -272,8 +279,8 @@ class ImageElement(VideoBase):
         glVertex2f(render_x, render_y)
         glEnd()
         
-        # Restore OpenGL state
-        glPopAttrib()
+        # Restore matrix state
+        glPopMatrix()
 
     def calculate_size(self) -> None:
         """Pre-calculate image box size including scaling, cropping, padding and styling.
