@@ -1,5 +1,5 @@
 from typing import Dict, Optional, Tuple, Any, Literal, Union, TypeVar
-from PIL import Image, ImageDraw
+from PIL import Image, ImageDraw, ImageFilter
 from .animation import Animation, AnimationManager, RepeatingAnimation
 
 # TypeVar for method chaining with inheritance
@@ -25,6 +25,7 @@ class VideoBase:
         border_color: Border color as RGB tuple
         border_width: Border width in pixels
         corner_radius: Corner radius for rounded corners
+        blur_strength: Blur strength (0 = no blur, higher values = more blur)
         crop_width: Crop width in pixels
         crop_height: Crop height in pixels
         crop_mode: Crop mode ('fill' or 'fit')
@@ -63,6 +64,9 @@ class VideoBase:
         # Corner radius settings
         self.corner_radius: float = 0
         
+        # Blur settings
+        self.blur_strength: float = 0
+        
         # Crop settings
         self.crop_width: Optional[int] = None
         self.crop_height: Optional[int] = None
@@ -83,6 +87,8 @@ class VideoBase:
         self.base_scale: float = 1.0  # Base scale before animations
         self.rotation: float = 0.0  # Rotation angle in degrees
         self.scale: float = 1.0  # Scale value
+        self.flip_horizontal: bool = False  # Horizontal flip flag
+        self.flip_vertical: bool = False  # Vertical flip flag
     
     def position(self: VideoBaseT, x: float, y: float, anchor: Optional[Literal['center', 'top-left', 'top-right', 'bottom-left', 'bottom-right']] = None) -> VideoBaseT:
         """Set position coordinates.
@@ -130,19 +136,23 @@ class VideoBase:
         """Get actual rendering position and size considering scale and other factors.
         
         Returns:
-            Tuple of (actual_x, actual_y, element_width, element_height)
+            Tuple of (actual_x, actual_y, scaled_width, scaled_height)
         """
         element_width = getattr(self, 'width', 0)
         element_height = getattr(self, 'height', 0)
         
-        # アンカーに基づく位置オフセットを計算
-        offset_x, offset_y = self._calculate_anchor_offset(element_width, element_height)
+        # スケールを適用した実際のサイズを計算
+        scaled_width = element_width * self.scale
+        scaled_height = element_height * self.scale
+        
+        # スケール適用後のサイズでアンカーに基づく位置オフセットを計算
+        offset_x, offset_y = self._calculate_anchor_offset(scaled_width, scaled_height)
         
         # 実際の描画位置を計算
         actual_x = self.x + offset_x
         actual_y = self.y + offset_y
         
-        return actual_x, actual_y, element_width, element_height
+        return actual_x, actual_y, scaled_width, scaled_height
     
     def set_duration(self: VideoBaseT, duration: float) -> VideoBaseT:
         """Set display duration in seconds.
@@ -256,6 +266,58 @@ class VideoBase:
         # サイズを再計算
         self.calculate_size()
         return self
+    
+    def set_blur(self: VideoBaseT, strength: float) -> VideoBaseT:
+        """Set blur strength.
+        
+        Args:
+            strength: Blur strength (0 = no blur, higher values = more blur)
+            
+        Returns:
+            Self for method chaining
+        """
+        self.blur_strength = max(0, strength)  # 負の値は0に補正
+        # テクスチャを再作成する必要がある
+        self.texture_created = False
+        return self
+    
+    def set_rotate(self: VideoBaseT, angle: float) -> VideoBaseT:
+        """Set rotation angle in degrees.
+        
+        Args:
+            angle: Rotation angle in degrees
+            
+        Returns:
+            Self for method chaining
+        """
+        self.rotation = angle
+        return self
+    
+    def set_flip(self: VideoBaseT, direction: Union[str, Literal['horizontal', 'vertical', 'both', 'none']] = 'horizontal') -> VideoBaseT:
+        """Set flip options for the element.
+        
+        Args:
+            direction: Flip direction - 'horizontal' (left-right), 'vertical' (up-down), 'both', or 'none'
+            
+        Returns:
+            Self for method chaining
+        """
+        if direction == 'horizontal':
+            self.flip_horizontal = True
+            self.flip_vertical = False
+        elif direction == 'vertical':
+            self.flip_horizontal = False
+            self.flip_vertical = True
+        elif direction == 'both':
+            self.flip_horizontal = True
+            self.flip_vertical = True
+        elif direction == 'none':
+            self.flip_horizontal = False
+            self.flip_vertical = False
+        else:
+            raise ValueError(f"Invalid flip direction: {direction}. Use 'horizontal', 'vertical', 'both', or 'none'")
+        
+        return self
 
     def _apply_border_and_background_to_image(self, img: Image.Image) -> Image.Image:
         """Apply background and border to an image.
@@ -351,6 +413,21 @@ class VideoBase:
         
         return img
     
+    def _apply_blur_to_image(self, img: Image.Image) -> Image.Image:
+        """Apply blur effect to an image.
+        
+        Args:
+            img: Source image to apply blur to
+            
+        Returns:
+            Blurred image
+        """
+        if self.blur_strength <= 0:
+            return img
+        
+        # PILのGaussianBlurフィルターを適用
+        return img.filter(ImageFilter.GaussianBlur(radius=self.blur_strength))
+    
     def _calculate_crop_dimensions(self, original_width: int, original_height: int) -> Tuple[int, int, int, int]:
         """Calculate crop scale and position.
         
@@ -677,6 +754,11 @@ class VideoBase:
         if animated_corner_radius is not None:
             properties['corner_radius'] = max(0, animated_corner_radius)
             
+        # ブラー強度のアニメーション
+        animated_blur_strength = self.animation_manager.get_animated_value('blur_strength', time, self.blur_strength)
+        if animated_blur_strength is not None:
+            properties['blur_strength'] = max(0, animated_blur_strength)
+            
         return properties
     
     def update_animated_properties(self, time: float) -> None:
@@ -699,6 +781,8 @@ class VideoBase:
             self.rotation = animated_props['rotation']
         if 'corner_radius' in animated_props:
             self.corner_radius = animated_props['corner_radius']
+        if 'blur_strength' in animated_props:
+            self.blur_strength = animated_props['blur_strength']
     
     def has_animations(self, time: Optional[float] = None) -> bool:
         """Check if element has animations.
